import datetime

import pandas as pd


def prepare_period(
    tz,
    date: datetime.datetime = None,
    month: datetime.datetime = None,
    year: datetime.datetime = None,
    period: tuple = None,
) -> tuple:
    """
    Prepare period for retrive data of wheather and calculate sun power.
    :param tz:  - time zone of geoposition of building
    :param date:
    :param month:
    :param year:
    :param period:
    :return: tuple (start , end) - begin and end of period.
    """
    if year and not month and not date:
        year = datetime.datetime(day=1, month=1, year=year)
        start = pd.Timestamp(year, tz=tz)
        end = start + pd.Timedelta(days=365)
    elif month and not date:
        if not year:
            year = datetime.datetime.now().year
        month = datetime.datetime(day=1, month=month, year=year)
        start = pd.Timestamp(month, tz=tz)
        end = start + pd.Timedelta(days=31)
    elif date:
        if not year:
            year = datetime.datetime.now().year
        if not month:
            month = datetime.datetime.now().month
        date = datetime.datetime(day=date, month=month, year=year)
        start = pd.Timestamp(date, tz=tz)
        end = start + pd.Timedelta(days=1)
    elif period:
        start = pd.Timestamp(period[0], tz=tz)
        end = pd.Timestamp(period[1], tz=tz)
    else:
        date = datetime.datetime.now()
        start = pd.Timestamp(date, tz=tz)
        end = start + pd.Timedelta(days=1)
    return start, end



class ThermalModel:
    """
    Class implements process of calculation of some model
    of thermal object which contains several thermal elements.
    As result you can take plots of temperatures of some thermal elements.
    """

    def __init__(self, name, **kwargs):
        """
        Initialization of model for calculations.
        :param name: string of name of model
        :param kwargs: some parameters
        """
        self.name = name
        self.elements = kwargs.get("elements", {})
        self.initial_conditions = kwargs.get("initial_conditions", {})
        self.start_element = kwargs.get("start_element", None)
        self.outside_elements = kwargs.get("outside", [])

    def show_schema(self):
        """ Shows schema of chain. """
        text = "%s ->" % self.start_element.name
        print(text)
        return

    def make_init_conditions(self) -> None:
        """Initialize conditions in elements."""
        for el, val in self.initial_conditions.items():
            self.elements[el].init_conditions(val)

    def start(self, count: int, dt: int, power: float, t_out: float) -> dict:
        """

        :param count: count of calculation
        :param dt: time for calculation (seconds)
        :param power: input power in first thermal element (Watt)
        :param t_out: temperature of last element
        :return:
            dict of data of temperatures of elements.
        """
        for el in self.outside_elements:
            el.temp = t_out
        for i in range(count):
            self.start_element.compute(power, dt)
        # TODO  make return data elements by dx for plots
        return



import math

import numpy as np
import pandas as pd
from pvlib.location import Location
from pvlib.modelchain import ModelChain
from pvlib.pvsystem import PVSystem
from pvlib.temperature import TEMPERATURE_MODEL_PARAMETERS
from trimesh import geometry, load, triangles

from . import settings

temp_model_pars = TEMPERATURE_MODEL_PARAMETERS["sapm"]["open_rack_glass_glass"]

properties_materials = {
    "adobe": {"transcalency": 0.04, "heat_capacity": 450.0, "density": 450.0},
    "water": {"transcalency": 0.599, "heat_capacity": 4182, "density": 998.29},
    "birch": {"transcalency": 0.15, "heat_capacity": 1250.0, "density": 700.0},
}


class Building:
    """
    Class implements methods for work with buildings for which calculate
    sun energy.
    Example: create building and test some it`s parameters.

    >>> text = 'o Cube\\n'
    >>> text += 'v 1.000000 1.000000 -1.000000\\n'
    >>> text += 'v 1.000000 0.000000 -1.000000\\n'
    >>> text += 'v 1.000000 1.000000 0.000000\\n'
    >>> text += 'v 1.000000 0.000000 0.000000\\n'
    >>> text += 'v 0.000000 1.000000 -1.000000\\n'
    >>> text += 'v 0.000000 0.000000 -1.000000\\n'
    >>> text += 'v 0.000000 1.000000 0.000000\\n'
    >>> text += 'v 0.000000 0.000000 0.000000\\n'
    >>> text += 's off\\n'
    >>> text += 'f 1/1/1 5/2/1 7/3/1 3/4/1\\n'
    >>> text += 'f 4/5/2 3/6/2 7/7/2 8/8/2\\n'
    >>> text += 'f 8/8/3 7/7/3 5/9/3 6/10/3\\n'
    >>> text += 'f 6/10/4 2/11/4 4/12/4 8/13/4\\n'
    >>> text += 'f 2/14/5 1/15/5 3/16/5 4/17/5\\n'
    >>> text += 'f 6/18/6 5/19/6 1/20/6 2/11/6\\n'
    >>> with open('test_file.obj','a') as file:\
            file.write(text)
    418
    >>> geo = {'latitude': 54.841426, 'longitude': 83.264479}
    >>> vertices = [[0,0,0],[1,0,0],[1,1,0],[0,1,0],[0,1,1],[0,0,1],[1,0,1],\
    [1,1,1]]
    >>> material = {'birch': {\
        'density': 700.0,\
        'transcalency': 0.15,\
        'heat_capacity': 1250.0}}
    >>> b = Building(mesh_file='test_file.obj',\
        geo=geo,\
        wall_thickness=0.3,\
        wall_material='birch',\
        properties_materials=material)
    >>> import os
    >>> os.remove('test_file.obj')
    >>> b.wall_thickness
    0.3
    >>> b.mesh.area
    6.0
    >>> round(b.mesh_inside.volume, 3)
    0.064
    >>> b.mesh.center_mass
    array([0. , 0. , 0.5])
    >>> b.floor_area_outside
    1.0
    >>> round(b.floor_area_inside, 3)
    0.16
    >>> b.windows['area']
    0.0
    >>> b.windows['area'] = 0.5
    >>> b.walls_area_outside
    4.5
    >>> round(b.walls_area_inside, 3)
    0.3
    >>> b.heat_accumulator['mass'] = 1
    >>> b.heat_accumulator['density'] = 1000
    >>> b.get_perimeter_floor('inside')
    1.6
    >>> round(b.area_mass_walls_inside, 2)
    0.2
    >>> round(b.volume_air_inside, 3)
    0.044
    >>> b.get_perimeter_floor('outside')
    4.0
    >>> round(b.area_mass_walls_outside, 3)
    1.7
    >>> import datetime, pytz
    >>> date = datetime.datetime(day=22, month=6, year=2020)

    """

    thermal_elements = [
        "mass",
        "room",
        "wall",
        "walls_mass",
        "floor",
        "windows",
        "outside",
        "fl_out",
    ]

    def __init__(
        self,
        mesh_file: str,
        geo: dict,
        wall_material: str = "adobe",
        wall_thickness: float = 0.3,
        start_temp_in: float = 20,
        power_heat_inside: float = 0,
        efficiency: float = 60,
        cover_material: str = None,
        heat_accumulator: dict = {"volume": 0.02, "material": "water"},
        **kwargs
    ) -> None:
        """ Initialize object of class Building. """
        self.mesh = load(mesh_file)
        self.__mesh_inside = None
        if not self.mesh.is_watertight:
            raise Exception("Mesh is not watertight", "Error")
        self.material = wall_material
        self.wall_thickness = wall_thickness
        self.current_temp = start_temp_in
        # power_heat_inside kWatt
        self.power_heat_inside = power_heat_inside * 1000
        self.efficiency = efficiency
        self.cover_material = cover_material
        self.dict_properties_materials = properties_materials
        self.wall_layers = kwargs.get("wall_layers", None)
        self.dict_power_inside = kwargs.get("dict_power_inside", None)
        self.dict_properties_materials = kwargs.get("properties_materials", properties_materials)
        self.ventilation_losses = kwargs.get("ventilation_losses", 0)
        self.heat_accumulator = heat_accumulator
        self.windows = kwargs.get("windows", {"therm_r": 0.0, "losses": 0.0, "area": 0.0})
        self.floor = kwargs.get("floor", {"material": self.material, "therm_r": 0, "area": 0, "layers": []})
        self.ceiling = kwargs.get("ceiling", {"material": self.material, "therm_r": 0, "area": 0, "layers": []},)
        self.extra_losses = kwargs.get("extra_losses", {})

        self.__centring()

        self.__correct_wall_thickness()

        self.weather_data = {}
        self.power_data = {}
        self.power_data_by_days = None
        self.location = Location(latitude=geo["latitude"], longitude=geo["longitude"],)
        self.pv = PVSystem(
            surface_tilt=45,
            surface_azimuth=180,
            module_parameters={"pdc0": 240, "gamma_pdc": -0.004},
            inverter_parameters={"pdc0": 240},
            temperature_model_parameters=temp_model_pars,
        )
        self.mc = ModelChain(self.pv, self.location, aoi_model="no_loss", spectral_model="no_loss")
        return

    def __correct_wall_thickness(self) -> None:
        """ Method for correct the wall thickness. """
        for base in self.mesh.bounding_box.primitive.extents:
            scale_factor = (base - self.wall_thickness * 2) / base
            if scale_factor < 0:
                self.wall_thickness = min(self.mesh.bounding_box.primitive.extents) / 2
        return

    def __centring(self):
        """ Method for centring of the mesh. """
        where_move = self.mesh.center_mass * -1.0
        self.mesh.apply_translation(where_move)
        where_move = self.mesh.center_mass
        z_move = abs(self.mesh.center_mass[2] - self.mesh.bounds[0][2])
        where_move[2] = z_move
        self.mesh.apply_translation(where_move)
        return

    @property
    def mesh_inside(self):
        """Get mesh of inside walls and floor of the house"""
        factors = []
        if self.__mesh_inside:
            return self.__mesh_inside
        for base in self.mesh.bounding_box.primitive.extents:
            scale_factor = (base - self.wall_thickness * 2) / base
            factors.append(scale_factor)
        if factors != [0.0, 0.0, 0.0]:
            matrix = np.diag(factors + [1.0])
            self.__mesh_inside = self.mesh.copy().apply_transform(matrix)
        return self.__mesh_inside

    @property
    def walls_area_inside(self):
        """Calculates area of walls inside the house"""
        area = self.mesh_inside.area - self.windows["area"] - self.floor_area_inside
        if area < 0:
            raise Exception("Area is null")
        return area

    @property
    def walls_area_outside(self):
        """Calculates area of walls outside the house"""
        area = self.mesh.area - self.windows["area"] - self.floor_area_outside
        if area < 0:
            raise Exception("Area is null")
        return area

    @property
    def face_normals(self) -> list:
        return self.mesh.face_normals

    @property
    def face_areas(self) -> list:
        return self.mesh.area_faces

    @property
    def volume_air_inside(self) -> float:
        """Calculates volume of the air inside the house"""
        return self.mesh_inside.volume - self.heat_accumulator_volume

    @property
    def floor_thickness(self) -> float:
        """Get floor thickness"""
        if "thickness" in self.floor and self.floor["thickness"]:
            return self.floor["thickness"]
        return self.wall_thickness

    @property
    def heat_accumulator_volume(self):
        """Get volume of heat accumulator"""
        if "volume" in self.heat_accumulator and self.heat_accumulator["volume"]:
            return self.heat_accumulator["volume"]
        if "material" in self.heat_accumulator and self.heat_accumulator["material"]:
            return self.get_prop(self.heat_accumulator["material"], "density") * self.heat_accumulator["mass"]
        return self.heat_accumulator["mass"] / self.heat_accumulator["density"]

    @property
    def area_mass_walls_inside(self) -> float:
        """Calculates area of the walls around the heat accumulator
        inside of the house."""
        if not self.heat_accumulator:
            return 0
        p = self.get_perimeter_floor("inside")
        h = self.heat_accumulator_volume / self.floor_area_inside
        return p * h

    @property
    def area_mass_walls_outside(self) -> float:
        """Calculates area of the walls around the heat accumulator outside of
         the house."""
        p = self.get_perimeter_floor("outside")
        h = self.heat_accumulator_volume / self.floor_area_inside
        th = self.wall_thickness
        if "thickness" in self.floor and self.floor["thickness"]:
            th = self.floor["thickness"]
        return (h + th) * p

    @property
    def floor_area_outside(self) -> float:
        """Calculates area floor outside of the house"""
        area = 0
        for norm, area_f in zip(self.mesh.face_normals, self.mesh.area_faces):
            if norm[2] == -1:
                area += area_f
        return area

    @property
    def floor_area_inside(self) -> float:
        """Calculates area floor inside of the house"""
        area = 0
        for norm, area_f in zip(self.mesh_inside.face_normals, self.mesh_inside.area_faces):
            if norm[2] == -1:
                area += area_f
        return area

    def get_perimeter_floor(self, where: str) -> float:
        """
        Calculate perimeter of floor

        :param where: 'inside' or 'outside'
        :return:
            float value of perimeter
        """
        if where == "outside":
            return self.mesh.section(plane_origin=self.mesh.bounds[0], plane_normal=[0, 0, 1]).length
        else:
            return self.mesh_inside.section(plane_origin=self.mesh_inside.bounds[0], plane_normal=[0, 0, 1]).length

    def get_efficient_angle(self, reflect_material: dict = None) -> float:
        """ Get angle for material. """
        ang = reflect_material[self.material]
        return ang

    def calc_reflect_power(self, power: float, sun_ang: float, cover_material: str = "polycarbonat") -> float:
        """
        Calculates
        power of reflection based on :
        https://majetok.blogspot.ru/2014/05/vid-na-teplicu.html.
        Returns: float of power of the reflection of material.

        """
        #    dict_kr_mat = {'polycarbonat':1.585,
        #                  'glass':1.4}
        #    dict_ki_mat = {'polycarbonat':0.82,
        #                    'glass':0.86}
        table_kr_mat = {
            "polycarbonat": {40: 0.04, 50: 0.06, 60: 0.1, 70: 0.18, 80: 0.4, 90: 1},
        }
        kr = 1
        for k, v in table_kr_mat[cover_material].items():
            if math.degrees(sun_ang) <= k:
                kr = v
                continue
        refl_power = power * kr
        return refl_power

    def get_pv_power_face(self, face_tilt: float, face_azimuth: float, face_area: float) -> float:
        """
        Get Irradiation from PVLIB.

        :param face_tilt: angle between normal of face and horizontal plane
        :param face_azimuth: angle between normal of face and north direction
        :param face_area: float value of area of face
        :return: pandas DataFrame with sun power of current period.
        """
        self.pv.surface_tilt = face_tilt
        self.pv.surface_azimuth = face_azimuth
        self.mc.run_model(self.weather_data)
        return self.mc.effective_irradiance * face_area * (self.efficiency / 100)

    def projection_on_flat(self, vector: tuple) -> tuple:
        """ get vector what is projection vector on the flat plane. """
        u = vector
        n = np.array([0, 0, 1])
        n_norm = np.sqrt(sum(n ** 2))
        proj_of_u_on_n = (np.dot(u, n) / n_norm ** 2) * n
        return u - proj_of_u_on_n

    def calc_sun_power_on_faces(self) -> None:
        """
        Calculates the power of sun on all faces of the building.

        :return: self
            changed self.power_data, self.power_data_by_days
        """
        dict_temp_data = {}
        face_indexes = []
        count_faces = 0

        if count_faces >= settings.COUNT_FACES_FOR_PARALLEL_CALC:
            # TODO Start parallels calc in actors model
            pass
        else:
            index = 0
            for face in self.mesh.faces:
                tri = [self.mesh.vertices[i].tolist() for i in face]
                face_area = triangles.area((tri,))
                face_normal = triangles.normals((tri,))[0][0]
                face_tilt = geometry.vector_angle((face_normal, (0, 0, 1)))
                face_normal_projection = self.projection_on_flat(face_normal)
                face_azimuth = geometry.vector_angle((face_normal_projection, (0, 1, 0)))
                sun_power_face = self.get_pv_power_face(face_tilt, face_azimuth, face_area,)
                face_seria = pd.Series(sun_power_face, index=self.weather_data.index,)
                dict_temp_data.update({index: face_seria})
                face_indexes.append(index)
                index += 1
        self.power_data = pd.DataFrame(dict_temp_data)
        fields = list(self.power_data)
        self.power_data["sum_solar_power"] = self.power_data[fields].sum(axis=1)
        self.power_data["maximum_solar_power"] = self.power_data[fields].max(axis=1)
        self.power_data["ind_face"] = self.power_data[fields].idxmax(axis=1)
        self.power_data_by_days = self.power_data["sum_solar_power"].resample("1D").mean()
        return

    def get_prop(self, material: str, prop: str) -> float:
        """
        Retrive a value of property for some materials.

        :param material: string of name material
        :param prop: string of name property
        :return: float of value property
        """
        if material not in self.dict_properties_materials:
            material = self.material
        if prop == "kappa":
            return self.dict_properties_materials[material]["transcalency"]
        return self.dict_properties_materials[material][prop]


if __name__ == "__main__":
    import doctest

    doctest.testmod()



#!/usr/bin/env python3
import math

import numpy as np


class ThermalElement:
    """
    Implements thermal element for thermal computation.
    Represents a point with heat capacity.
    Change of temperature of this point depends on sum of input and
    output energy and heat capacity (output energy is negative).
    Several elements can be connected into a chain of elements.
    Output energy depends on temperature of current element, temperature of
    next element in chain, and thermal resistance between each other.
    An element can have several elements of output enegry.
    Second and further elements must have area of input face (square meters)
    and input coefficient of transcalency on input face.

    Also element may be represented as a wall with a variable area
    and with variable thermal resistance on each increment of thickness, dx.
    Computation is implemented in single dimension, dx (meters).
    All computations are performed for increments of time, dt.
    Example: compute temperature of 1 cubic meter of water in 1 hour with
             1 kW of power applied:
    >>> e = ThermalElement(\
        name='cube_water',\
        temp0=0,\
        density=997,\
        heat_capacity=4180,\
        volume=1\
    )
    >>> e.count_layers
    1
    >>> e.compute(q_enter=1000, dt=3600)
    >>> round(e.temp, 3)
    0.864
    >>>
    Example: calculate temperature of inside face of wall of birch
    with dx = 0.01 m and external power of 1 kW.
    Result of test calculated manually.
    >>> e = ThermalElement(\
        name='birch_wall',\
        temp0=20.0,\
        density=700.0,\
        heat_capacity=1250.0,\
        dx=0.01,\
        thickness=0.20,\
        kappa=0.15,\
        area_inside=1.0,\
        area_outside=1.1\
    )
    >>> e.count_layers
    20
    >>> e.dTx_list
    [20.0, 20.0, 20.0, 20.0, 20.0, 20.0, 20.0, 20.0, 20.0, 20.0, 20.0, 20.0,\
 20.0, 20.0, 20.0, 20.0, 20.0, 20.0, 20.0, 20.0]
    >>> e.get_loss_dx(0)
    0.0
    >>> e.compute(q_enter=1000, dt=1)
    >>> round(e.dTx_list[0], 3)
    20.109
    >>> round(e.dTx_list[1], 3)
    20.0
    >>> round(e.get_loss_dx(0),3)
    1.714
    >>> e.compute(q_enter=1000, dt=1)
    >>> round(e.dTx_list[0], 3)
    20.218
    >>> round(e.dTx_list[1], 4)
    20.0002
    >>>
    Example element which implementing  thin layer between two areas
    >>> e = ThermalElement(\
        name='glass',\
        temp0=20.0,\
        area_inside=1.0,\
        input_alpha=23,\
    )
    >>> e.calc_loss_input_q(25.0)
    115.0
    """

    def __init__(self, name, temp0=None, density=None, heat_capacity=None, volume=None, **kwargs):
        """Initialize object of class Element"""
        self.name = name
        self.temp = temp0
        self.round = 5
        self.density = density
        self.heat_capacity = heat_capacity
        self.volume = volume
        self.thickness = kwargs.get("thickness", None)
        self.kappa = kwargs.get("kappa", 0.04)
        self.layers = kwargs.get("layers", {})
        self.dx = kwargs.get("dx", None)
        self.area_inside = kwargs.get("area_inside", None)
        self.area_outside = kwargs.get("area_outside", None)
        self.input_alpha = kwargs.get("input_alpha", None)
        self.by_avegare = kwargs.get("by_average", True)

        self.branches_loss = []
        self.counter = 0
        self.count_layers = 1
        self.dTx_list = [self.temp]
        if self.thickness and self.dx:
            self.count_layers = int(self.thickness / self.dx)
            self.dTx_list = list(np.ones(self.count_layers) * self.temp)
        self.k_area = None
        self.area_average = None
        if self.area_inside and self.area_outside and self.area_outside > self.area_inside:
            if self.by_avegare:
                self.area_average = (self.area_outside + self.area_inside) / 2
            else:
                self.k_area = (math.sqrt(self.area_outside) - math.sqrt(self.area_inside)) / self.thickness

    def init_conditions(self, val):
        """Reduction to initial conditions"""
        new_list = []
        for t in self.dTx_list:
            new_list.append(val)
        self.dTx_list = new_list
        self.temp = self.dTx_list[0]
        return

    def __get_area_dx(self, iterator):
        """
        Calculates area of loss power for current dx.

        :param iterator: number of dx
        :return:
            Float value of area
        """
        if self.by_avegare:
            return self.area_average
        else:
            d_linear = math.sqrt(self.area_inside) + iterator * self.dx * self.k_area
            return d_linear * d_linear

    def __get_kappa_dx(self, iterator):
        """
        Calculate the kappa if it depends of dx

        :param iterator: number of dx
        :return:
            Float value of thermal resistance of dx layer.
        """
        if self.kappa:
            return self.kappa

    def __get_cm_dx(self, iterator):
        """
        Calculates heat capacity of mass on current dx
        :param iterator: number of dx
        :return:
            Float value of heat capacity
        """
        a = self.density * self.heat_capacity
        if self.count_layers == 1:
            return self.volume * a
        return self.dx * self.__get_area_dx(iterator) * a

    def calc_loss_input_q(self, t_in: float) -> float:
        """Calculates loss energy between current and previous elements"""
        return self.input_alpha * self.area_inside * (t_in - self.temp)

    def get_loss_dx(self, iterator):
        """
        Defines loss energy from current element on dx or from all
        element if it represent in calculation as a point.
        q_loss = alpha*area_branch*(T_current - T_branch)

        :param iterator: number of dx, 0 if element as a point
        :return: Float value of all loss power
        """
        temp1 = self.dTx_list[iterator]
        temp2 = self.dTx_list[iterator + 1]
        if temp1 == temp2:
            return 0.0
        area = self.__get_area_dx(iterator + 1)
        if area > self.area_outside:
            area = self.area_outside
        kappa = self.__get_kappa_dx(iterator)
        q_loss = (area * (temp1 - temp2)) / (self.dx / kappa)

        return q_loss

    def calc_temp(self, q_enter: float, q_loss: float, iterator: int, dt: float) -> None:
        """
        Calculates the dT on dt of current point (dx) of element.
        If element represent as a point then calculates.
        Tdx = Tdx0 + (q_enter - q_loss)/cmdx

        :param q_enter: enter power from previouse element or source of power
        :param q_loss: total power loss from current point dx
        :param iterator: number of current dx
        :param dt: range of time for calculate
        :return:
            Nothing returns but change temperature in list of
            temperatures by dx in the current point
        """
        cm_dx = self.__get_cm_dx(iterator)
        qcm = q_enter - q_loss
        dT = dt * qcm / cm_dx
        if dT:
            self.dTx_list[iterator] = self.dTx_list[iterator] + dT
        return

    def compute(self, q_enter: float, dt: float) -> None:
        """
        Start of calculate temperature of element if
        it represent as a point or calculate of all
        temperatures by dx if element has the dx parameter

        :param q_enter: input power
        :param dt: range of time
        :return: change self.temp parameter in the end of calculation
        """
        if not self.heat_capacity or not self.density:
            return
        for i in range(0, self.count_layers):
            q_loss = 0
            if (i + 1) == self.count_layers:
                for branch in self.branches_loss:
                    q = branch.calc_loss_input_q(self.dTx_list[i])
                    branch.compute(q, dt)
                    q_loss += q
            else:
                q_loss = self.get_loss_dx(i)
            self.calc_temp(q_enter, q_loss, i, dt)
            q_enter = q_loss

        self.temp = round(self.dTx_list[0], self.round)


if __name__ == "__main__":
    import doctest

    doctest.testmod()



import os

import matplotlib.pyplot as plt
import mpld3
import pandas as pd


def as_file(pd_data: pd.DataFrame, type_file: str = "csv", path: str = "output") -> None:
    """ Export results to file. """

    file_path = os.path.join(path, "data.%s" % type_file)
    with open(file_path, "w", newline="") as file:
        if type_file == "csv":
            file.write(pd_data.to_csv())
        else:
            file.write(pd_data.to_json(orient="split"))
            pass
    return file_path


def as_html(pd_data: pd.DataFrame, output_file_dir: str) -> None:
    """ Create HTML page with graphics. """
    fig = plt.figure()
    ax = fig.subplots()
    ax.plot(pd_data)
    file_obj = os.path.join(output_file_dir, "plots.html")
    mpld3.save_html(fig, file_obj)



import datetime

import pandas as pd
import pytz
from pvlib.forecast import GFS

from .building import Building
from .helpers import prepare_period
from .thermal_process import ThermalProcess


class Calculation:
    """
    Class implements methods for calculate of the solar power
    what you can take on faces of the building.
    As result you can get html page with graphics.
    Also you can export data in file CSV or JSON.
    """

    def __init__(self, tz: str, geo: dict, building: Building):
        """ Initialize object for calculate sun power. """
        self.progress = 0
        self.geo = geo
        self.tz = pytz.timezone(tz)
        self.pd_data_for_export = None
        self.building = building

    def compute(
        self,
        date: datetime.datetime = None,
        month: datetime.datetime = None,
        year: datetime.datetime = None,
        period: tuple = None,
        with_weather: bool = True,
    ) -> None:
        """ proxy method for prepare period and calculations. """
        start, end = prepare_period(tz=self.tz, date=date, month=month, year=year, period=period)
        return self.start_calculation(start, end, with_weather=with_weather)

    def __get_weather(self, start: pd.Timestamp, end: pd.Timestamp) -> pd.DataFrame:
        """
        Get weather data for period.
        :param start: - pd.Timestamp, begin of period
        :param end: - pd.Timestamp, end of period
        :return: pd.DataFrame,
            Column names are: ``ghi, dni, dhi``
        """
        fx_model = GFS()
        return fx_model.get_processed_data(self.geo["latitude"], self.geo["longitude"], start, end)

    def __get_clear_sky(self, start: pd.Timestamp, end: pd.Timestamp, model: str = "ineichen") -> pd.DataFrame:
        """
        Get sun data of irradiation of sun without weather.
        :param start: - pd.Timestamp, begin of period
        :param end: - pd.Timestamp, end of period
        :param model: - str, The clear sky model to use. Must be one of
            'ineichen', 'haurwitz', 'simplified_solis'.
        :return: pd.DataFrame,
            Column names are: ``ghi, dni, dhi``
        """
        period = pd.date_range(start=start, end=end, freq="1h", tz=self.tz)
        return self.building.location.get_clearsky(period, model=model)

    def start_calculation(self, start: pd.Timestamp, end: pd.Timestamp, with_weather: bool = True) -> None:
        """ Start calculations. """
        get_weather = self.__get_clear_sky
        if with_weather:
            get_weather = self.__get_weather
        self.building.weather_data = get_weather(start, end)
        self.building.calc_sun_power_on_faces()
        thermal_process = ThermalProcess(
            t_start=20, building=self.building, variant="heat_to_mass", for_plots=["mass", "room"],
        )
        self.pd_data_for_export = thermal_process.run_process()
        return self.pd_data_for_export


if __name__ == "__main__":
    import doctest

    doctest.testmod()



from solarhouse.thermal_element import ThermalElement


def test_cube_water():
    """
    Test cube water as a thermal point.
    Result of test calculated manually.
    """
    e = ThermalElement(name="cube_water", temp0=0, density=997, heat_capacity=4180, volume=1)
    assert e.count_layers == 1
    e.compute(1000, 3600)
    assert round(e.temp, 3) == 0.864


def test_wall_birch():
    """
    Example calculate of wall from birch with dx = 0.01
    meter and 1 kW power coming to inside area of element.
    All result for test calculated manually.
    In this test made two circle of calculation of all 20 point of wall.
    """
    e = ThermalElement(
        name="birch_wall",
        temp0=20.0,
        density=700.0,
        heat_capacity=1250.0,
        dx=0.01,
        thickness=0.20,
        kappa=0.15,
        area_inside=1.0,
        area_outside=1.1,
    )
    assert e.count_layers == 20
    assert e.dTx_list == [
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
        20.0,
    ]
    assert e.get_loss_dx(0) == 0.0
    e.compute(1000, 1)
    assert round(e.dTx_list[0], 3) == 20.109
    assert round(e.dTx_list[1], 3) == 20.0
    assert round(e.get_loss_dx(0), 3) == 1.714
    e.compute(1000, 1)
    assert round(e.dTx_list[0], 3) == 20.218
    assert round(e.dTx_list[1], 4) == 20.0002


def test_thin_layer():
    """Example element which implementing  thin layer between two areas."""
    e = ThermalElement(name="glass", temp0=20.0, area_inside=1.0, input_alpha=23,)
    assert e.calc_loss_input_q(25.0) == 115.0



import pandas as pd

from .building import Building
from .thermal_element import ThermalElement
from .thermal_model import ThermalModel


class ThermalProcess:
    """
    Class implements all calculations of thermal processes in a house.
    There are three main models of a house:
    1. All solar power comes into massive body (water tank, concrete
       plate, etc.) in the house with respect to efficient coefficient
       of water solar collector.
    2. All solar power heats up air inside the house with respect
       efficient coefficient of air solar collector.
    3. All solar power heats up walls through a glass dome.
    """

    def __init__(
        self, t_start: float, building: Building, variant: str = "heat_to_mass", for_plots: list = ["mass"],
    ) -> None:
        """
        Initialize item of thermo calculation.
        c - ;
        1,007 - Specific heat capacity of air (wikipedia)
        1,1839 - Density of air (wikipedia)
        There are creating all main thermo elements of the building:
        1. Massive object inside the building
        (It is model of any massive objects: water boiler, hot floor ,etc.)
        2. Air inside the building.
        3. Walls (without windows).
        4. Windows (if area of windows set in building).
        5. Floor.
        6. Area walls around the massive object.
        7. Outside  - temperature of air around the building.
        8. Floor outside - temperature of air under the floor
        under the insulation
        9. Glass dome around the building.
        This elements can be combined to three variant
        (power to massive object, power to air, power to walls).
        dx for non-homogeneous elements is in meters.
        """
        self.count = 0
        self.seconds = 60
        self.building = building
        self.t_start = t_start
        self.elements_for_plots = for_plots
        self.sun_power_data = self.building.power_data["sum_solar_power"].resample("1h").interpolate()
        self.weather_data = self.building.weather_data["temp_air"].resample("1h").interpolate()

        self.alpha_room = 1 / 0.13
        self.alpha_out = 1 / 0.04

        self.dx = 0.005  # meters
        self.heat_accumulator_volume = self.building.heat_accumulator["volume"]
        self.heat_accumulator_density = self.building.get_prop(self.building.heat_accumulator["material"], "density")
        if not self.building.heat_accumulator["volume"]:
            self.heat_accumulator_volume = self.building.heat_accumulator["mass"] / self.heat_accumulator_density

        mass = ThermalElement(
            name="mass",
            temp0=self.t_start,
            density=self.heat_accumulator_density,
            heat_capacity=4183,
            volume=self.building.heat_accumulator["volume"],
            area_inside=self.building.floor_area_inside,
            input_alpha=self.alpha_room,
        )
        room = ThermalElement(
            name="room",
            temp0=self.t_start,
            density=1.27,
            heat_capacity=1007,
            volume=self.building.volume_air_inside,
            area_inside=self.building.floor_area_inside,
            input_alpha=self.alpha_room,
        )
        windows = ThermalElement(
            name="windows",
            temp0=-5,
            area_inside=self.building.windows["area"],
            input_alpha=1 / self.building.windows["therm_r"],
        )
        floor = ThermalElement(
            name="floor",
            temp0=18,
            area_inside=self.building.floor_area_inside,
            area_outside=self.building.floor_area_outside,
            dx=self.dx,
            thickness=self.building.floor_thickness,
            kappa=self.building.get_prop(self.building.floor["material"], "kappa"),
            density=self.building.get_prop(self.building.floor["material"], "density"),
            heat_capacity=self.building.get_prop(self.building.floor["material"], "heat_capacity"),
            input_alpha=self.alpha_room,
        )
        walls = ThermalElement(
            name="walls",
            temp0=self.t_start,
            area_inside=self.building.walls_area_inside,
            area_outside=self.building.walls_area_outside,
            dx=self.dx,
            thickness=self.building.wall_thickness,
            kappa=self.building.get_prop(self.building.material, "kappa"),
            density=self.building.get_prop(self.building.material, "density"),
            heat_capacity=self.building.get_prop(self.building.material, "heat_capacity"),
            input_alpha=self.alpha_room,
        )

        walls_mass = ThermalElement(
            name="walls_mass",
            temp0=self.t_start,
            dx=self.dx,
            area_inside=self.building.area_mass_walls_inside,
            area_outside=self.building.area_mass_walls_outside,
            thickness=self.building.wall_thickness,
            kappa=self.building.get_prop(self.building.material, "kappa"),
            density=self.building.get_prop(self.building.material, "density"),
            heat_capacity=self.building.get_prop(self.building.material, "heat_capacity"),
            input_alpha=self.alpha_room,
        )
        outside = ThermalElement(
            name="outside", temp0=-5, area_inside=self.building.walls_area_outside, input_alpha=self.alpha_out,
        )
        fl_outside = ThermalElement(
            name="fl_out",
            temp0=self.building.floor["t_out"],
            area_inside=self.building.floor_area_outside,
            input_alpha=self.alpha_out,
        )

        walls.branches_loss = [outside]
        walls_mass.branches_loss = [outside]
        floor.branches_loss = [fl_outside]

        self.model = ThermalModel(name=variant)
        self.model.elements = {
            "mass": mass,
            "room": room,
            "wall": walls,
            "walls_mass": walls_mass,
            "floor": floor,
            "windows": windows,
            "outside": outside,
            "fl_out": fl_outside,
        }
        self.model.initial_conditions = {
            "mass": self.t_start,
            "room": self.t_start,
            "wall": self.t_start,
            "walls_mass": self.t_start,
            "floor": self.t_start,
        }
        self.model.outside_elements = [outside]

        if variant == "heat_to_mass":
            mass.branches_loss = [room, floor, walls_mass]
            room.branches_loss = [windows, walls]
            self.model.start_element = mass
        elif variant == "heat_to_air":
            room.branches_loss = [windows, walls, mass]
            mass.branches_loss = [floor, walls_mass]
            self.model.start_element = room
        elif variant == "heat_to_walls":
            pass

    def run_process(self) -> dict:
        """
        Start main calculation process.
        In the end of process it show a plots of temperatures

        :return: dict data of elements in house for plots.
        """
        self.seconds = 60 * 60
        dt = 3
        count_dt = int(self.seconds / dt)
        # pd_for_plot = pd.DataFrame(self.sun_power_data)
        pd_for_plot = pd.DataFrame(self.weather_data)
        # pd_for_plot.insert(1, "temp_air", self.weather_data)
        dict_for_plot = {}
        self.model.make_init_conditions()
        for name, el in self.model.elements.items():
            print(name, ": ", el.temp)
        for el_name in self.elements_for_plots:
            dict_for_plot.update({el_name: []})
        for index in self.sun_power_data.index:
            # TODO make progress status
            for el in self.elements_for_plots:
                dict_for_plot[el].append(self.model.elements[el].temp)
            sun = self.sun_power_data[index]
            t_out = self.weather_data[index]
            self.model.start(count=count_dt, dt=dt, power=sun, t_out=t_out)
        count = 0
        for k in dict_for_plot.keys():
            count += 1
            seria = pd.Series(dict_for_plot[k], self.sun_power_data.index)
            pd_for_plot.insert(count, k, seria)
        return pd_for_plot



import os
import setuptools

from src.solarhouse.version import version

this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setuptools.setup(
    name="solarhouse",  # Replace with your own username
    version=version,
    author="Yaric Pisarev",
    author_email="yaricp@gmail.com",
    description="package for calculate solar power what you can get\
                on surfaces of building by solar heat collectors",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yaricp/py-solarhouse",
    package_dir={'solarhouse': 'src/solarhouse'},
    packages=['solarhouse'],
    install_requires=[
        'numpy',
        'scipy',
        'trimesh',
        'pvlib',
        'pandas',
        'matplotlib',
        'mpld3',
        'shapely',
        'jinja2',
        'netCDF4',
        'siphon',
        'tables',
    ],
    dependency_links=['https://github.com/Unidata/netcdf4-python'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



import os
import filecmp

from solarhouse.building import Building
from solarhouse.calculation import Calculation
import solarhouse.export as export


def test_main(mesh_file_path, tmpdir):
    tz = "Asia/Novosibirsk"
    geo = {
        "latitude": 54.841426,
        "longitude": 83.264479,
    }
    calc = Calculation(
        tz=tz,
        geo=geo,
        building=Building(
            mesh_file=mesh_file_path,
            geo=geo,
            wall_material="adobe",
            wall_thickness=0.3,
            start_temp_in=20.0,
            power_heat_inside=0.0,
            efficiency=75,  # TODO: percents or fractions of 1?
            heat_accumulator={"volume": 0.032, "material": "water"},
            windows={"area": 0.3, "therm_r": 5.0},
            floor={"area": 1.0, "material": "adobe", "thickness": 0.2, "t_out": 4.0},
        ),
    )
    data_frame = calc.compute(date=22, month=12, year=2019, with_weather=False)

    output_dir = tmpdir

    export.as_file(data_frame, "csv", output_dir)
    res_file = os.path.join(output_dir, "data.csv")
    ref_res_file = os.path.join("test/ref_files", "data.csv")
    assert filecmp.cmp(res_file, ref_res_file)

    export.as_html(data_frame, output_dir)
    assert os.path.exists(os.path.join(output_dir, "plots.html"))



def test_thickness(building):
    assert building.wall_thickness == 0.3


def test_area(building):
    assert building.mesh.area == 6.0


def test_inside_volume(building):
    assert round(building.mesh_inside.volume, 3) == 0.064


def test_center_mass(building):
    assert building.mesh.center_mass[0] == 0.0
    assert building.mesh.center_mass[1] == 0.0
    assert building.mesh.center_mass[2] == 0.5


def test_floor_area_outside(building):
    assert building.floor_area_outside == 1.0


def test_floor_area_inside(building):
    assert round(building.floor_area_inside, 3) == 0.16


def test_walls_area(building):
    assert building.walls_area_outside == 5.0
    assert round(building.walls_area_inside, 3) == 0.8


def test_windows_area(building):
    assert building.windows["area"] == 0.0
    building.windows["area"] = 0.5
    assert building.walls_area_outside == 4.5
    assert round(building.walls_area_inside, 3) == 0.3


def test_perimeter_floor(building):
    building.heat_accumulator["mass"] = 1
    building.heat_accumulator["density"] = 1000
    assert building.get_perimeter_floor("inside") == 1.6
    assert round(building.area_mass_walls_inside, 2) == 0.2
    assert round(building.volume_air_inside, 3) == 0.044
    assert building.get_perimeter_floor("outside") == 4.0
    assert round(building.area_mass_walls_outside, 3) == 1.7



import datetime
import pytz
import pandas as pd

from solarhouse.helpers import prepare_period

tz = pytz.timezone("Asia/Novosibirsk")


def test_year():
    start, end = prepare_period(tz=tz, year=2019)
    assert type(start) == pd.Timestamp
    assert type(end) == pd.Timestamp
    assert start == pd.Timestamp("2019-01-01 00:00:00+0700", tz="Asia/Novosibirsk")
    assert end == pd.Timestamp("2020-01-01 00:00:00+0700", tz="Asia/Novosibirsk")


def test_month():
    start, end = prepare_period(tz=tz, month=7)
    assert type(start) == pd.Timestamp
    assert type(end) == pd.Timestamp
    assert start == pd.Timestamp("2020-07-01 00:00:00+0700", tz="Asia/Novosibirsk")
    assert end == pd.Timestamp("2020-08-01 00:00:00+0700", tz="Asia/Novosibirsk")
    start, end = prepare_period(tz=tz, month=7, year=2017)
    assert type(start) == pd.Timestamp
    assert type(end) == pd.Timestamp
    assert start == pd.Timestamp("2017-07-01 00:00:00+0700", tz="Asia/Novosibirsk")
    assert end == pd.Timestamp("2017-08-01 00:00:00+0700", tz="Asia/Novosibirsk")


def test_date():
    start, end = prepare_period(tz=tz, date=17)
    assert type(start) == pd.Timestamp
    assert type(end) == pd.Timestamp
    month_now = datetime.datetime.now().month
    assert start == pd.Timestamp("2020-%s-17 00:00:00+0700" % month_now, tz="Asia/Novosibirsk")
    assert end == pd.Timestamp("2020-%s-18 00:00:00+0700" % month_now, tz="Asia/Novosibirsk")
    start, end = prepare_period(tz=tz, date=17, month=5)
    assert type(start) == pd.Timestamp
    assert type(end) == pd.Timestamp
    assert start == pd.Timestamp("2020-05-17 00:00:00+0700", tz="Asia/Novosibirsk")
    assert end == pd.Timestamp("2020-05-18 00:00:00+0700", tz="Asia/Novosibirsk")
    start, end = prepare_period(tz=tz, date=17, year=2017)
    assert type(start) == pd.Timestamp
    assert type(end) == pd.Timestamp
    assert start == pd.Timestamp("2017-%s-17 00:00:00+0700" % month_now, tz="Asia/Novosibirsk")
    assert end == pd.Timestamp("2017-%s-18 00:00:00+0700" % month_now, tz="Asia/Novosibirsk")
    start, end = prepare_period(tz=tz, date=17, month=5, year=2017)
    assert type(start) == pd.Timestamp
    assert type(end) == pd.Timestamp
    assert start == pd.Timestamp("2017-05-17 00:00:00+0700", tz="Asia/Novosibirsk")
    assert end == pd.Timestamp("2017-05-18 00:00:00+0700", tz="Asia/Novosibirsk")


def test_period():
    period = (
        datetime.datetime(day=12, month=6, year=2017),
        datetime.datetime(day=6, month=12, year=2017),
    )
    start, end = prepare_period(tz=tz, period=period)
    assert type(start) == pd.Timestamp
    assert type(end) == pd.Timestamp
    assert start == pd.Timestamp("2017-06-12 00:00:00+0700", tz="Asia/Novosibirsk")
    assert end == pd.Timestamp("2017-12-06 00:00:00+0700", tz="Asia/Novosibirsk")


def test_empty_date():
    start, end = prepare_period(tz=tz)
    assert type(start) == pd.Timestamp
    assert type(end) == pd.Timestamp
    assert start.date() == pd.Timestamp(datetime.datetime.now(), tz="Asia/Novosibirsk")
    assert end.date() == pd.Timestamp(datetime.datetime.now() + datetime.timedelta(days=1), tz="Asia/Novosibirsk")
